# -*- coding: utf-8 -*-
"""
Tooling package that provides a set of different tools.
"""
from orkgnlp.common.tools.downloader import download
from orkgnlp.common.tools.loader import load

__all__ = ["download", "load"]
